      subroutine init_dmrg()
      use mod_dmrg
#ifdef EigenExa
      use eigen_libs
#endif
      implicit none
      include 'mpif.h'
      integer :: isp(2),i,ib,ista,iend,isize,ise1,ise2,mhx,ix1,ix2,
     &           jsta,jend,lsta(2),lend(2)
      integer,allocatable :: iwk(:)
#ifdef cmplx_v
      complex(8),allocatable :: hsm(:,:)
#else
      real(8),allocatable :: hsm(:,:)
#endif

! --- SET PARAMETERS
      call init_parameters()
      call def_parameters()
      call read_operators()
      call read_Hamiltonian()
      call read_parameters()
      call init_operators()

! --- NUMBER OF DMRG STEPS
      ndmrg_i=(1+nsite-nse(1)-nse(2))/nadd   !+1 for odd number sites system
      ndmrg_s=max(2*(nsite-(ls_l+ls_r+naddf)),0)

      if(id_tdmrg.eq.1) then !tDMRG
        ndmrg_dt=ndmrg_s
        ndmrg_f=ndmrg_s*nsweep_g+ndmrg_dt*nsweep_t
      else !DMRG,DDMRG
        ndmrg_f=ndmrg_s*nsweep_g
      end if

      ndmrg=ndmrg_i+ndmrg_f

! --- INITIALIZE NUMBER OF SPIN
      limup=max(min(1,inup),int(((dble(inup)+1.d-8)/dble(nsite))*4.d0))
      limdw=max(min(1,indw),int(((dble(indw)+1.d-8)/dble(nsite))*4.d0))

c --- physical quantities etc.
      allocate(phys(4))
c ----- 1:truncation number
      allocate(phys(1)%isw(0:nsweep))
c ----- 2:energy
      allocate(phys(2)%dsw(0:nsweep))
c ----- 3:entanglement entropy
      allocate(phys(3)%dsw(0:nsweep))
c ----- 4:truncation error
      allocate(phys(4)%dsw(0:nsweep))

!$OMP PARALLEL WORKSHARE
      phys(1)%isw=0
      phys(2)%dsw=0.d0
      phys(3)%dsw=0.d0
      phys(4)%dsw=0.d0
!$OMP END PARALLEL WORKSHARE

! --- SETTING MPI PARALLELIZATION
! ----- INTERACTIONS
      if(index(pal%interact,'A').ne.0.or.
     &   index(pal%interact,'a').ne.0) then
        call split_comm(newcomm_dmrg,myid_dmrg,np_dmrg,1,
     &                  newcomm_int,myid_int,np_int,
     &                  newcomm_tm,myid_tm,np_tm)
      elseif(index(pal%interact,'N').ne.0.or.
     &       index(pal%interact,'n').ne.0) then
        call split_comm(newcomm_dmrg,myid_dmrg,np_dmrg,np_dmrg,
     &                  newcomm_int,myid_int,np_int,
     &                  newcomm_tm,myid_tm,np_tm)
      elseif(index(pal%interact,'D').ne.0.or.
     &       index(pal%interact,'d').ne.0) then
        call split_comm(newcomm_dmrg,myid_dmrg,np_dmrg,
     &                  np_dmrg/pal%n_interact,
     &                  newcomm_int,myid_int,np_int,
     &                  newcomm_tm,myid_tm,np_tm)
      elseif(index(pal%interact,'L').ne.0.or.
     &       index(pal%interact,'l').ne.0) then
        call fact2(np_dmrg,isp)
        call split_comm(newcomm_dmrg,myid_dmrg,np_dmrg,minval(isp),
     &                  newcomm_int,myid_int,np_int,
     &                  newcomm_tm,myid_tm,np_tm)
      else
        call fact2(np_dmrg,isp)
        call split_comm(newcomm_dmrg,myid_dmrg,np_dmrg,maxval(isp),
     &                  newcomm_int,myid_int,np_int,
     &                  newcomm_tm,myid_tm,np_tm)
      end if

! ----- ADDED SITES
!!!      if(mod(np_tm,nfr).eq.0) then
!!!        i=np_tm/nfr
!!!      else
!!!        i=np_tm
!!!      end if
      i=np_tm
      call split_comm(newcomm_tm,myid_tm,np_tm,i,
     &                newcomm_add,myid_add,np_add,
     &                newcomm_bm,myid_bm,np_bm)

      call para_range(1,nfr,np_add,myid_add,ista_add,iend_add)
      allocate(idest_add(nfr),iwk(nfr))
      iwk=0
      do i=ista_add,iend_add
        iwk(i)=myid_add
      enddo
      call mpi_allreduce(iwk,idest_add,nfr,mpi_integer,mpi_sum,
     &                   newcomm_add,ierr)
      deallocate(iwk)

! ----- BLOCK & MATRIX
      call split_comm_bm()
      do mhx=1,nh
        do ise1=1,2
          if(myid_bk.eq.0) then
            if(id_space.eq.0) then
              block(ise1,1,mhx)%isbk=1; block(ise1,1,mhx)%iebk=1
            else
              block(ise1,1,mhx)%isbk=1; block(ise1,1,mhx)%iebk=nfr
            end if
          else
            block(ise1,1,mhx)%isbk=1; block(ise1,1,mhx)%iebk=0
          end if

          do ise2=1,2
            do ib=1,nbk
              if(id_space.eq.0) then
                call para_ranges(1,nfr,np_(ise2),myid_(ise2),
     &                           ista,iend,isize)
              else
                call para_ranges(1,1,np_(ise2),myid_(ise2),
     &                           ista,iend,isize)
              end if
              block(ise1,1,mhx)%ista0(ib,ise2)=ista
              block(ise1,1,mhx)%iend0(ib,ise2)=iend
              block(ise1,1,mhx)%isize0(ib,ise2)=isize
              block(ise1,1,mhx)%ista(ib,ise2)=ista
              block(ise1,1,mhx)%iend(ib,ise2)=iend
              block(ise1,1,mhx)%isize(ib,ise2)=isize
            enddo
          enddo

c ----- TRANSLATION MATRIX
          if(id_mapping.eq.1) then
            if(id_space.eq.0) then
              ise2=1+mod(ise1,2)
              call para_range(1,nfr,np_(ise1),myid_(ise1),ista,iend)
              call para_range(1,nfr,np_(ise2),myid_(ise2),jsta,jend)
              allocate(block(ise1,1,mhx)%b(1)%tm(ista:iend,jsta:jend))
              block(ise1,1,mhx)%b(1)%tm=zero
              do ix1=max(ista,jsta),min(iend,jend)
                block(ise1,1,mhx)%b(1)%tm(ix1,ix1)=one
              enddo
            else
              if(myid_sys+myid_env.eq.0) then
                do ib=1,nfr
                  allocate(block(ise1,1,mhx)%b(ib)%tm(1,1))
                  block(ise1,1,mhx)%b(ib)%tm(1,1)=one
                enddo
              else
                do ib=1,nfr
                  allocate(block(ise1,1,mhx)%b(ib)%tm(0,0))
                enddo
              end if
            end if
          end if
        enddo
      enddo

      if(myid_dmrg.eq.key_rank) then
        write(iso,*)
        write(iso,'(1x,a)') '# information of DMRG calculation'
#ifdef cmplx_v
        write(iso,'(2x,a)') '(kind of variable): complex(8)'
#else
        write(iso,'(2x,a)') '(kind of variable): real(8)'
#endif
        write(iso,*)
        write(iso,'(1x,a)') '# information of MPI parallelization'
        write(iso,'(2x,a,I0)') '(interactions): ',np_int
!!!        write(iso,'(2x,a,I0)') '(added sites):  ',np_add
        write(iso,'(2x,a,I0)') '(spin space):   ',np_bk
!!!        write(iso,'(2x,a,I0)') '(matrix vec.):  ',np_vec
        write(iso,'(2x,a,I0)') '(matrix sys.):  ',np_sys
        write(iso,'(2x,a,I0)') '(matrix env.):  ',np_env
      end if

#ifdef sclpk
      call eigen_init_sclpk() !initialize Scalapack
#endif
#ifdef EigenExa
      call eigen_init(newcomm_mx) !initialize EigenExa
#endif

      if(id_space.eq.0) then !fix Hamiltonian for parallelizations
        do mhx=1,nh
          call para_range(1,nfr,np_vec,myid_vec,jsta,jend)
          do ise1=1,2
            ise2=1+mod(ise1,2)
            call para_range(1,nfr,np_(ise1),myid_(ise1),
     &                      lsta(1),lend(1))
            call para_range(jsta,jend,np_(ise2),myid_(ise2),
     &                      lsta(2),lend(2))
            allocate(hsm(nfr,nfr))
            hsm=block(ise1,1,mhx)%b(1)%hm
            deallocate(block(ise1,1,mhx)%b(1)%hm)
            allocate(block(ise1,1,mhx)%b(1)%hm(lsta(ise1):lend(ise1),
     &                                         lsta(ise2):lend(ise2)))
            do ix2=lsta(ise2),lend(ise2)
              do ix1=lsta(ise1),lend(ise1)
                block(ise1,1,mhx)%b(1)%hm(ix1,ix2)=hsm(ix1,ix2)
              enddo
            enddo
            deallocate(hsm)
          enddo
        enddo
      end if

      if(myid_dmrg.eq.0) open(99,file=file_ene)

      return
      end
