      subroutine lanczos(v0,neg,mab)
      use mod_dmrg
      implicit none
      include 'mpif.h'
      real(8),allocatable :: a(:),b(:),e(:),vt(:,:)
      integer :: neg,id,mab,nab,nl,ill,ieg(neg),iv
      integer(8) :: imem
      real(8) :: ax,ev(neg)
      type(avec) :: v0(isbk1:iebk1,neg)

! --- INITIALIZATION
      if(lim_mem.eq.0.or.8*mab*ndimsp/np_dmrg.le.lim_mem) then
        id=1
      else
        id=0
      end if

      ev=0.d0

      if(myid_dmrg.eq.key_rank) then
        write(iso,'(1x,a)') '# Lanczos method'
        write(iso,'(10x,a,5x,a)') '< energy >','< diff. >'
      end if

      allocate(v2(isbk0:iebk0,2))
      call lz_acv(v2(isbk0:iebk0,1),isbk0,iebk0,0,imem)
      call lz_acv(v2(isbk0:iebk0,2),isbk0,iebk0,1,imem)

      if(id.eq.1) then
        allocate(vk(isbk0:iebk0,mab))
      end if

      if(iebk0.ge.isbk0) then
        call lz_init(v0(isbk1:iebk1,1),isbk1,iebk1,
     &               v2(isbk0:iebk0,1),isbk0,iebk0)
      end if

      allocate(a(mab),b(mab),e(mab),vt(mab,mab))

! --- LANCZOS ITERATION
      b(1)=1.d0
      do nl=1,mab-1
        if(id.eq.1) call lz_save(nl)
        call matrix_vec(1,v2(:,1),v2(:,2),one)
        call lz_oth(0,nl)   !(id,nl):orthonormalization(id=1)
        call ipv(v2(isbk1:iebk1,1),v2(isbk1:iebk1,2),isbk1,iebk1,a(nl))

        call lz_ex(a(nl))
        call ipv(v2(isbk1:iebk1,1),v2(isbk1:iebk1,1),isbk1,iebk1,
     &           b(nl+1))
        b(nl+1)=dsqrt(b(nl+1))
        call lz_newv(2,-b(nl+1))
        call lz_newv(1,1.d0/b(nl+1))

        if(mod(nl,4).eq.0.or.nl.le.2) then !check point of convergence
          call diagola(mab,a,b,e,vt,nl,1,ill)
          if(myid_dmrg.eq.key_rank) then
            if(nl.gt.1) then
              write(iso,'(1x,I4,1x,f14.7,1x,1Pe14.5)') 
     &          nl,e(1),abs(e(1)-ev(1))
            else
              write(iso,'(1x,I4,1x,f14.7,4x,a)') nl,e(1),' --------- '
            end if
          end if
          if(nl.eq.2) pvw=abs(e(1)-ev(1))

          if(allocated(ddmrg)) then
            if(mdDMRG.ne.0.and.isweep.gt.ddmrg(1)%nsg-1) then
              if(abs(e(1)-ev(1)).le.ecv.and.nl.gt.4) then
                exit
              else
                call lz_eck(e,nl,ev,ieg,neg)
              end if
            else
              if(abs(e(1)-ev(1)).le.ecv.and.nl.ne.1) then
                exit
              else
                call lz_eck(e,nl,ev,ieg,neg)
              end if
            end if
          else
            if(abs(e(1)-ev(1)).le.ecv.and.nl.ne.1.and.
     &         abs(e(1)).gt.1.d-8) then
              exit
            else
              call lz_eck(e,nl,ev,ieg,neg)
            end if
          end if

        end if
      enddo   ! END OF LANCZOS ITERATION

      nab=nl
      if(nab.eq.mab) then
        if(id.eq.1) call lz_save(mab)
        call matrix_vec(1,v2(:,1),v2(:,2),one)
        call lz_oth(0,nl)
        call ipv(v2(isbk1:iebk1,1),v2(isbk1:iebk1,2),isbk1,iebk1,a(mab))
        call lz_ex(a(mab))
        call diagola(mab,a,b,e,vt,nab,2,ill)
        if(myid_dmrg.eq.key_rank) then
          write(iso,'(1x,I4,1x,f14.7,1x,1Pe14.5)') 
     &      mab,e(1),abs(e(1)-ev(1))
        end if
      else
        call diagola(mab,a,b,e,vt,nab,2,ill)
      end if
      call lz_eck(e,nl,ev,ieg,neg)

      if(myid_dmrg.eq.key_rank) then
        write(iso,'(2x,a,I0)') '(Lanczos truncation number) = ',nab
      end if

      if(iebk0.ge.isbk0) then
        call lz_init(v0(isbk1:iebk1,1),isbk1,iebk1,
     &               v2(isbk0:iebk0,1),isbk0,iebk0)
      end if

C ---- EIGEN STATE
      if(id.eq.1) then
        do iv=1,neg
          call lz_eigen(v0(:,iv),iv,ieg,nab,mab,vt)
        enddo
      else
        call vclr(v2(isbk0:iebk0,2),isbk0,iebk0,zero)
        do nl=1,nab   !LANCZOS ITERATION 2
          call lz_state(v0(:,1),1,ieg,nl,vt,mab,v2(isbk0:iebk0,1))

          if(nl.eq.nab) exit
          call matrix_vec(1,v2(:,1),v2(:,2),one)
          call ipv(v2(isbk1:iebk1,1),v2(isbk1:iebk1,2),isbk1,iebk1,
     &             a(nl))
          call lz_ex(a(nl))
          call ipv(v2(isbk1:iebk1,1),v2(isbk1:iebk1,1),isbk1,iebk1,
     &             b(nl+1))
          b(nl+1)=dsqrt(b(nl+1))
          call lz_newv(2,-b(nl+1))
          call lz_newv(1,1.d0/b(nl+1))
        enddo   !EMD OF LANCZOS ITERATION 2
      end if
      emin=e(1)
      emax=e(nab)
      emin_min=min(emin,emin_min)
      emax_max=max(emax,emax_max)
      deallocate(a,b,e,vt,v2)

! --- OUTPUT DATA
      do iv=1,neg
        call lz_norm(v0(:,iv),ax)
        if(myid_dmrg.eq.key_rank) then
          write(iso,'(1x,a,I0)') '# state ',iv
          write(iso,'(2x,a,f18.12)') '(energy) = ',ev(iv)
          write(iso,'(2x,a,f18.12)') '(energy/sites) = ',
     &                                 ev(iv)/dble(nse(1)+nse(2)+nadd)
          write(iso,'(2x,a,1Pe12.5)') '(norm) = ',ax
        end if
        if(abs(1.d0-ax).gt.1.d-16) then
          call normalize(v0(:,iv))
        end if
      enddo

      if(iphys.eq.1) then
        phys(2)%dsw(icphys)=ev(1)
      end if
      ene_t=ev(1)

      if(id.eq.1) deallocate(vk)

      return
      end
