      module mod_dmrg_parameters
      integer :: nsite,  !number of sites
     &           ndmrg,  !number of DMRG steps
     &           ndmrg_i,!number of DMRG steps for infinite
     &           ndmrg_s,!number of DMRG steps for one sweeping
     &           ndmrg_f,!number of DMRG steps for finite
     &           idmrg,  !counter of DMRG steps
     &           nsweep, !number of sweep in finite DMRG
     &           nadd,   !number of added site(s)
     &           naddf,  !number of added site(s)
     &           nas,    !number of added site(s) in system
     &           nae,    !number of added site(s) in environment
     &           nfr,    !degree of freedom at a site
     &           nop,    !number of operators
     &           limup,  !number of upspin at each DMRG step
     &           limdw,  !number of downspin at each DMRG step
     &           inup,   !number of upspin
     &           indw,   !number of downspin
     &           nbk,    !number of spin-spaces
     &           nit,    !number of kinds of interactions (off-diagonal)
     &           mab0,   !maximum truncation number of Lanczos,
     &           mab1,   !maximum truncation number of Lanczos (final),
     &           mts,    !DMRG truncation number
     &           iso,    !unit number of standard out,
     &           mdDMRG, !infinit(0) or finite(1:-->,2:<--)
     &           nh2,    !number of terms in Hamiltonian (2)
     &           nh4,    !number of terms in Hamiltonian (4)
     &           iphys,  !flag of calculation of physical quantities
     &           icphys, !counter of physical quantites
     &           ls_l,   !left edge of calculating region
     &           ls_r,   !right edge of calculating region
     &           iwf,    !flag of wavefunction transformation
     &           iftp,   !kind of definition of Fermion sign
     &           ntargd, !number of target states of double precision
     &           mabcv,  !truncation number of RPM
     &           isweep, !present sweep number
     &           nhcf,   !number of coefficient for each interactions
     &           mhcf,   !number of kinds of coefficients of interactions
     &           id_spin,   !1:spin system
     &           id_es,  !show entanglement spectrum
     &           num_eigen, !number of calculating eigenstate from ground state
     &           mpi_kv,  !kind of variable for MPI communications
     &           id_fc,   !1:A(q)-<A(q)>
     &           itdmrg,  !scenario number of tDMRG
     &           nsweep_g,!number of sweep for ground state
     &           nsweep_t,!number of sweep for time evolution
     &           iswp_ev, !counting number of sweep for an event
     &           num_ev,  !number of events
     &           id_hop,  !1:make hopping block Hamiltonian
     &           ns_ech,  !number of sweep for each time step
     &           mh,      !Hilbert space number
     &           nh,      !number of Hilbert space
     &           ndmrg_ex,!number of extra DMRG step
     &           ndmrg_dt,
     &           id_fast  !1:fast mode

      integer(1) :: id_tdmrg,  !1:tDMRG calculation
     &              id_ddmrg,  !1:DDMRG calculation
     &              if_tdmrg,  !flag of tDMRG
     &              if_ddmrg,  !flag of DDMRG
     &              ich_time,  !flag of changing time
     &              id_current, !1:calculating current
     &              id_doublon, !1:calculating doublon
     &              id_spinless,!1:spinless
     &              id_multi,   !1:multi target in tDMRG
     &              id_mapping  !1:using mapping space

      integer(8) :: ndimsp,   !dimension of superblock
     &              lim_mem,  !upper limitation of memory in Lanczos method
     &              id_space  !spin space

      real(8) :: ecv,
     &           etr,
     &           emin,
     &           emax,
     &           emin_min,
     &           emax_max,
     &           pwt,   !RPM weight
     &           psf,   !RPM shift
     &           pvw,   !weight for DM in the case of single added site
     &           ene_t,
     &           ee_t,
     &           err_t,
     &           w_nz,
     &           ptime,  !present time in tDMRG
     &           dlim_int   !lower limit of strength of interaction

#ifdef cmplx_v
      complex(8),parameter :: pi=cmplx(3.141592653589793d0,0.d0,kind=8),
     &                        zero=cmplx(0.d0,0.d0,kind=8),
     &                        one=cmplx(1.d0,0.d0,kind=8),
     &                        two=cmplx(2.d0,0.d0,kind=8)
#else
      real(8),parameter :: pi=3.141592653589793d0,
     &                     zero=0.d0,
     &                     one=1.d0,
     &                     two=2.d0
#endif
      complex(8) :: ps !LASER

      character(1) :: boundary
      character(1) :: chmm(3)
      character(32) :: file_param,   !parameter
     &                 file_input,
     &                 file_stout,   !standard out
     &                 file_ddmrg,
     &                 file_preset,
     &                 file_posit,
     &                 file_phys,
     &                 file_mag,
     &                 file_ene,
     &                 file_scenario,
     &                 file_current,
     &                 file_doublon,
     &                 file_es,
     &                 file_qphys,
     &                 file_phys_op,
     &                 file_dphys_op,
     &                 file_qphys_op

      character(8) :: dir_out  !directry name of output

      type parallel
      character(8) :: interact,
     &                block,
     &                matrix_env,
     &                vector
      integer :: n_interact,
     &           n_block,
     &           n_matrix_env,
     &           n_vector
      end type parallel
      type(parallel) :: pal

      type hamiltonian
      integer :: nop
      integer,allocatable :: ihs(:)
#ifdef cmplx_v
      complex(8),
#else
      real(8),
#endif
     &  allocatable :: vh(:)
      end type hamiltonian
      type(hamiltonian),allocatable :: hamil(:),hamil0(:)

      type physq
      real(8),allocatable :: dsw(:)
      integer,allocatable :: isw(:)
      end type physq
      type(physq),allocatable :: phys(:)

      type ddmrg_para
      real(8) :: wmin,
     &           wmax,
     &           qx,
     &           qy,
     &           sigma
      integer :: nw,   !# of omega points
     &           nsg,  !# of sweepings for ground state
     &           nop   !# of operators
      integer,allocatable :: iop(:) 
      end type ddmrg_para
      type(ddmrg_para),allocatable :: ddmrg(:)

      type tdmrg_para
      integer :: ks,   !kind of scenario
     &           ns,   !total number of time step
     &           is    !number of point in one time step
      real(8) :: ts    !time step
      real(8),allocatable :: pt(:) !parameters of each scenario
      end type tdmrg_para
      type(tdmrg_para),allocatable :: tdmrg(:)

      integer,allocatable :: iud(:,:),   !number of up and down spin at added site
     &                       nch(:,:),   !changing spin by corresponding operators
     &                       kch(:),
     &                       kud(:),
     &                       ihm(:),
     &                       ad(:,:),
     &                       nse(:),
     &                       ndmx(:),
     &                       iht(:,:),
     &                       ihcf(:),    !coefficient parameter for interactions
     &                       nfrx(:)     !degree of freedom at a site

      real(8),allocatable :: op(:,:,:),
     &                       time(:),
     &                       times(:),
     &                       ht(:),
     &                       fsi(:),
     &                       fmag(:,:),
     &                       cmag(:)

#ifdef cmplx_v
      complex(8),allocatable :: opz(:,:,:)
#endif

      character(16),allocatable :: name_op(:)

      end module mod_dmrg_parameters
