      subroutine read_Hamiltonian()
      use mod_dmrg
      implicit none
      include 'mpif.h'
      integer :: ix1,ix2,ios,nhx,is,ih,ich,id,id_mag
      integer,parameter :: nbuf=1000000
      character(256) :: para

! --- INITIALIZATION
      id=1
      id_mag=0

! --- HAMILTONIAN
! --- AB-INITIO CASE (VASP etc.) (name.input)
      call use_input(id)
      if(id.eq.0) return

! --- READ 'file_preset'
      if(myid_dmrg.eq.key_rank) then
        ich=1
        do while(ich.eq.1)
          ich=0   !for checking number of sites
          do ix1=1,2   !ix1=1:counting, ix1=2:input data
            open(10,file=file_preset,status='old',err=100)

            if(ix1.eq.2) then
              allocate(hamil(nhx))
              do ix2=1,nhx
                allocate(hamil(ix2)%vh(nhcf))
                hamil(ix2)%vh=zero
              enddo
            else
              if(allocated(hamil)) deallocate(hamil)
              allocate(i2v(nbuf))
            end if

            nh2=0; nh4=0; nhx=0; ios=0; ih=2
            do while(ios.ge.0)
              read(10,'(a)',iostat=ios) para

              if(ios.lt.0) exit
              if(index(para,'!').ne.0) cycle
              if(len_trim(para).eq.0) cycle
              call cpara(para,is,ih,nhx)
              if(is.eq.1) then
                cycle
              elseif(is.eq.2) then
                exit
              end if

              if(ih.eq.2) then !2-body
                nh2=nh2+1
                call rop2(para,nhx)
              elseif(ih.eq.4) then !4-body
                nh4=nh4+1
                call rop4(para,nhx)
              elseif(ih.eq.11.and.ix1.eq.2) then !magnetic field
                id_mag=1
                call read_mag(para)
              end if
            enddo

            nhx=nh2+nh4 !total number of interaction terms
            close(10)
          enddo !count & read

          deallocate(i2v)

          do ix1=1,nhx
            if(maxval(hamil(ix1)%ihs).gt.nsite) then
              nsite=maxval(hamil(ix1)%ihs)
              ich=1
            end if
          enddo

        enddo
      end if

! --- BROADCAST PARAMETERS
      call mpi_bcast(nsite,1,mpi_integer,key_rank,newcomm_dmrg,ierr)
      call mpi_bcast(nhx,1,mpi_integer,key_rank,newcomm_dmrg,ierr)
      call mpi_bcast(nh2,1,mpi_integer,key_rank,newcomm_dmrg,ierr)
      call mpi_bcast(nh4,1,mpi_integer,key_rank,newcomm_dmrg,ierr)

! --- PARALLELIZATION of 'nh4'
      allocate(icth4(np_dmrg),idsh4(np_dmrg))
      call para_info(nh2+1,nh2+nh4,np_dmrg,myid_dmrg,newcomm_dmrg,
     &               icth4,idsh4)

      if(myid_dmrg.ne.key_rank) then
        allocate(hamil(nhx))
        do ix1=1,nh2
          hamil(ix1)%nop=2
         allocate(hamil(ix1)%ihs(2))
        enddo
        do ix1=nh2+1,nhx
          hamil(ix1)%nop=4
          allocate(hamil(ix1)%ihs(4))
        enddo
        if(id_mag.eq.1) then
          allocate(fmag(3,nsite))
        end if
      end if


! --- BROAD CAST HAMILTONIAN
      do ix1=1,nh2
        call mpi_bcast(hamil(ix1)%ihs,2,mpi_integer,key_rank,
     &                 newcomm_dmrg,ierr)
        if(.not.allocated(hamil(ix1)%vh)) allocate(hamil(ix1)%vh(nhcf))
        call mpi_bcast(hamil(ix1)%vh,nhcf,mpi_kv,key_rank,newcomm_dmrg,
     &                 ierr)
      enddo
      do ix1=nh2+1,nhx
        call mpi_bcast(hamil(ix1)%ihs,4,mpi_integer,key_rank,
     &                 newcomm_dmrg,ierr)
        if(.not.allocated(hamil(ix1)%vh)) allocate(hamil(ix1)%vh(nhcf))
        call mpi_bcast(hamil(ix1)%vh,nhcf,mpi_kv,key_rank,newcomm_dmrg,
     &                 ierr)
      enddo

#ifndef cmplx_v
      if(myid_dmrg.eq.key_rank) then
        open(10,file='preset_trans.dat')
        if(nh2.gt.0) write(10,'(a)') '# 2' !2-body
        do ix1=1,nh2
          write(10,'(I0,1x,I0)',advance='no') hamil(ix1)%ihs(1:2)
          if(nhcf.eq.1) then
            write(10,'(1x,f16.10)',advance='no') hamil(ix1)%vh(1)
          else
            do ix2=1,nhcf
              write(10,'(1x,f8.4)',advance='no') hamil(ix1)%vh(ix2)
            enddo
          end if
          write(10,*)
        enddo

        if(nh4.gt.0) write(10,'(a)') '# 4' !4-body
        do ix1=nh2+1,nhx
          write(10,'(I0,1x,I0,1x,I0,1x,I0,1x,f16.10)')
     &      hamil(ix1)%ihs(1:4),hamil(ix1)%vh(1)
        enddo

        if(id_mag.eq.1) then !magnetic field
          write(10,'(a)') '# M'
          do ix1=1,nsite
            write(10,'(I0,1x,f10.6,1x,f10.6,1x,f10.6)')
     &        ix1,fmag(1:3,ix1)
          enddo
        end if
        close(10)
      end if
#endif

      call mpi_bcast(id_mag,1,mpi_integer,key_rank,
     &               newcomm_dmrg,ierr)

      if(id_mag.eq.1) then
        if(.not.allocated(fmag)) then
          allocate(fmag(3,nsite))
        end if

        call mpi_bcast(fmag,3*nsite,MPI_REAL8,key_rank,
     &                 newcomm_dmrg,ierr)
      end if

      if(myid_dmrg.eq.key_rank) then
        write(iso,'(1x,a)') '# system information'
        write(iso,'(2x,a,I0)') '(number of sites) = ',nsite
        write(iso,*)
        if(id_mag.eq.1) then
          write(iso,'(1x,a)') '# magnetic field'
          write(iso,'(1x,a)') '(site)   ( h_z )   ( h_y )   ( h_z )'
          do ix1=1,nsite
            write(iso,'(1x,I5,1x,3f10.5)') ix1,fmag(1:3,ix1)
          enddo
          write(iso,*)
        end if
      end if

      return

  100 continue

      call mpi_abort(newcomm_dmrg,ierr)

      stop
      end
